/*
 * Copyright (C) 2015 Stratio (http://stratio.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
(function () {
  'use strict';

  angular
    .module('webApp')
    .factory('PolicyModelFactory', PolicyModelFactory);
  PolicyModelFactory.$inject = ['CubeModelFactory', 'fragmentConstants'];

  function PolicyModelFactory(CubeModelFactory, fragmentConstants) {
    var policy = {};
    var finalJSON = {};
    var template = {};
    var error = {};

    function initPolicy() {
      policy.name = "";
      policy.description = "";
      policy.input = {};
      policy.outputs = [];
      policy.transformations = [];
      policy.cubes = [];
      policy.streamTriggers = [];
      ///* Reset policy advanced settings to be loaded from template automatically */
      delete policy.checkpointPath;
      delete policy.sparkStreamingWindowNumber;
      delete policy.sparkStreamingWindowTime;
      delete policy.storageLevel;
      delete policy.rawDataEnabled;
      delete policy.rawDataPath;
    }

    function setPolicy(inputPolicyJSON) {
      policy.id = inputPolicyJSON.id;
      policy.name = inputPolicyJSON.name;
      policy.description = inputPolicyJSON.description;
      policy.sparkStreamingWindow = inputPolicyJSON.sparkStreamingWindow;
      policy.remember = inputPolicyJSON.remember;
      policy.storageLevel = inputPolicyJSON.storageLevel;
      policy.checkpointPath = inputPolicyJSON.checkpointPath;
      policy.rawDataEnabled = (inputPolicyJSON.rawData.enabled == "true");
      policy.rawDataPath = inputPolicyJSON.rawData.path;
      policy.transformations = inputPolicyJSON.transformations;
      policy.cubes = setCubes(inputPolicyJSON.cubes);
      policy.streamTriggers = setStreamTriggers(inputPolicyJSON.streamTriggers);
      formatAttributes();
      var policyFragments = separateFragments(inputPolicyJSON.fragments);
      policy.input = policyFragments.input;
    }

    function formatAttributes() {
      var sparkStreamingWindow = policy.sparkStreamingWindow.split(/([0-9]+)/);
      policy.sparkStreamingWindowNumber = Number(sparkStreamingWindow[1]);
      policy.sparkStreamingWindowTime = sparkStreamingWindow[2];
      delete policy.sparkStreamingWindow;
      if (policy.remember) {
        var rememberField = policy.remember.split(/([0-9]+)/);
        policy.rememberNumber = Number(rememberField[1]);
        policy.rememberTime = rememberField[2];
      }
      delete policy.remember;
    }

    function setStreamTriggers(streamTriggers) {
      var formattedStreamTriggers = [];
      for (var i = 0; i < streamTriggers.length; ++i) {
        var trigger = streamTriggers[i];
        var overLast = trigger.overLast.split(/([0-9]+)/);
        trigger.overLastNumber = Number(overLast[1]);
        trigger.overLastTime = overLast[2];
        delete trigger.overLast;
        formattedStreamTriggers.push(trigger);
      }
      return formattedStreamTriggers;
    }

    function setCubes(cubes) {
      var formattedCubes = [];
      for (var i = 0; i < cubes.length; ++i) {
        formattedCubes.push(CubeModelFactory.getParsedCube(cubes[i]));
      }
      return formattedCubes;
    }

    function setTemplate(newTemplate) {
      template = newTemplate;
    }

    function getTemplate() {
      return template;
    }

    function separateFragments(fragments) {
      var result = {};
      var input = null;
      var fragment = null;

      for (var i = 0; i < fragments.length; ++i) {
        fragment = fragments[i];
        if (fragment.fragmentType == fragmentConstants.INPUT) {
          input = fragment;
        }
      }

      result.input = input;
      return result;
    }


    function getCurrentPolicy() {
      if (Object.keys(policy).length == 0)
        initPolicy();
      return policy;
    }

    function resetPolicy() {
      initPolicy();
    }

    function getAllModelOutputs() {
      var allModelOutputs = [];
      var models = policy.transformations;
      var outputs = [];
      var modelOutputs, output = null;
      for (var i = 0; i < models.length; ++i) {
        modelOutputs = models[i].outputFields;
        for (var j = 0; j < modelOutputs.length; ++j) {
          output = modelOutputs[j];
          if (outputs.indexOf(output) == -1) {
            allModelOutputs.push(output.name);
          }
        }
      }
      return allModelOutputs;
    }

    function getFinalJSON() {
      return finalJSON;
    }

    function setFinalJSON(json) {
      return finalJSON = json;
    }

    function isValidSparkStreamingWindow() {
      var valid = true;
      if (policy.streamTriggers && policy.sparkStreamingWindowNumber > 0) {
        var i = 0;
        while (valid && i < policy.streamTriggers.length) {
          valid = valid && ((policy.streamTriggers[i].overLastNumber % policy.sparkStreamingWindowNumber) == 0);
          ++i;
        }
      }
      return valid;
    }

    function getError() {
      return error;
    }

    function setError(text, type, subErrors) {
      error.text = text;
      error.type = type;
      error.subErrors = subErrors;
    }

    return {
      setPolicy: setPolicy,
      setTemplate: setTemplate,
      getTemplate: getTemplate,
      getCurrentPolicy: getCurrentPolicy,
      resetPolicy: resetPolicy,
      getAllModelOutputs: getAllModelOutputs,
      getFinalJSON: getFinalJSON,
      setFinalJSON: setFinalJSON,
      isValidSparkStreamingWindow: isValidSparkStreamingWindow,
      getError: getError,
      setError: setError
    }
  }

})();


